CREATE EXTENSION pgcrypto;
CREATE TABLE IF NOT EXISTS users (
  -- Alternative if pgcrypto is not available:
  -- overlay(overlay(md5(random()::text || ':' || clock_timestamp()::text) placing '4' from 13) placing '8' from 17)::uuid
  id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
  orgid INT,
  orgkey TEXT,
  loginid INT,
  phone TEXT,
  phone_verified TEXT,
  phone_verification_code TEXT,
  environment TEXT,
  user_role TEXT,
  -- JSON { notification_id: BOOL }
  subscriptions JSONB,
  config JSONB,
  persona TEXT,
  createdat TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  updatedat TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP
);
-- ALTER TABLE users ADD COLUMN IF NOT EXISTS orgkey TEXT;
CREATE TABLE IF NOT EXISTS notifications (
  id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
  userid UUID references users (id),
  createdby UUID references users (id),
  type TEXT,
  title TEXT,
  content TEXT,
  action JSONB,
  fields JSONB,
  dismissed BOOLEAN,
  readtime TIMESTAMP WITH TIME ZONE,
  createdat TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  updatedat TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP
);
CREATE TABLE IF NOT EXISTS notifications_history (
    id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
    incident_id TEXT,
    event_id TEXT,
    event_time TEXT,
    alert_rule_id TEXT,
    alert_rule_name TEXT,
    threat_id TEXT,
    sha256hash TEXT,
    device_id TEXT,
    comment TEXT,
    orgid INT,
    loginid INT,
    userid UUID references users (id),
    delivery_phone_number TEXT,
    delivery_webhook_url TEXT,
    delivery_email TEXT,
    delivery_email_sent TIMESTAMP WITH TIME ZONE,
    delivery_email_status TEXT,
    delivery_in_app_sent TIMESTAMP WITH TIME ZONE,
    delivery_in_app_status TEXT,
    delivery_webhook_sent TIMESTAMP WITH TIME ZONE,
    delivery_webhook_status TEXT,
    createdat TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
    updatedat TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP
);

-- Somehow the primary key doesn't seem to auto create indices..
CREATE INDEX IF NOT EXISTS user_by_id ON notifications (id);
CREATE INDEX IF NOT EXISTS notifications_by_id ON notifications (id);
CREATE INDEX IF NOT EXISTS users_by_env ON users (environment);
CREATE INDEX IF NOT EXISTS users_by_orgkey ON users (orgkey);
CREATE INDEX IF NOT EXISTS notifications_by_createdby ON notifications (createdby);
CREATE INDEX IF NOT EXISTS notification_history_by_userid ON notifications_history (userid);
CREATE INDEX IF NOT EXISTS user_by_env_oridg_loginid ON users (environment, orgid, loginid);
CREATE INDEX IF NOT EXISTS user_by_orgkey_loginid ON users (orgkey, loginid);
CREATE INDEX IF NOT EXISTS notifications_by_user ON notifications (userid);
CREATE INDEX IF NOT EXISTS unread_notifications_by_user ON notifications (userid, readtime, dismissed);
CREATE INDEX IF NOT EXISTS undismissed_notifications_by_user ON notifications (userid, dismissed);

-- remember to add pre-upgrade hook in migration.yaml when the script is updated
